@@ -23,49 +23,35 @@
 */
 
 #include <config.h>
-#include <ctype.h>
-
 #include "eel-background.h"
 
-#include <math.h>
-
-#include <stdio.h>
-#include "eel-glib-extensions.h"
-
-#include <gtk/gtksignal.h>
+#include "eel-background-style.h"
 #include "eel-gdk-extensions.h"
 #include "eel-gdk-pixbuf-extensions.h"
+#include "eel-glib-extensions.h"
 #include "eel-gnome-extensions.h"
-#include "eel-background-canvas-group.h"
-#include "eel-lib-self-check-functions.h"
 #include "eel-gtk-macros.h"
+#include "eel-lib-self-check-functions.h"
 #include "eel-string.h"
-
-#include <libgnomeui/gnome-canvas.h>
-#include <libgnomeui/gnome-canvas-util.h>
-
+#include <gtk/gtkselection.h>
+#include <gtk/gtksignal.h>
 #include <libart_lgpl/art_rgb.h>
+#include <libgnomecanvas/gnome-canvas-util.h>
+#include <libgnomecanvas/gnome-canvas.h>
+#include <libgnomevfs/gnome-vfs-async-ops.h>
+#include <math.h>
+#include <stdio.h>
 
-static void     eel_background_initialize_class          (gpointer       klass);
-static void     eel_background_initialize                (gpointer       object,
+static void     eel_background_class_init          (gpointer       klass);
+static void     eel_background_init                (gpointer       object,
 							  gpointer       klass);
-static void     eel_background_destroy                   (GtkObject     *object);
-static void     eel_background_draw_flat_box             (GtkStyle      *style,
-							  GdkWindow     *window,
-							  GtkStateType   state_type,
-							  GtkShadowType  shadow_type,
-							  GdkRectangle  *area,
-							  GtkWidget     *widget,
-							  char          *detail,
-							  int            x,
-							  int            y,
-							  int            width,
-							  int            height);
+static void     eel_background_finalize                  (GObject       *object);
 static void     eel_background_start_loading_image       (EelBackground *background,
-							  gboolean       emit_appearance_change);
+							  gboolean       emit_appearance_change,
+							  gboolean       load_async);
 static gboolean eel_background_is_image_load_in_progress (EelBackground *background);
 
-EEL_DEFINE_CLASS_BOILERPLATE (EelBackground, eel_background, GTK_TYPE_OBJECT)
+EEL_CLASS_BOILERPLATE (EelBackground, eel_background, GTK_TYPE_OBJECT)
 
 enum {
 	APPEARANCE_CHANGED,
@@ -75,6 +61,8 @@ enum {
 	LAST_SIGNAL
 };
 
+#define	RESET_BACKGROUND_MAGIC_IMAGE_NAME "reset.png"
+
 static guint signals[LAST_SIGNAL];
 
 struct EelBackgroundDetails {
@@ -107,59 +95,57 @@ struct EelBackgroundDetails {
 };
 
 static void
-eel_background_initialize_class (gpointer klass)
+eel_background_class_init (gpointer klass)
 {
-	GtkObjectClass *object_class;
+	GObjectClass *object_class;
 	EelBackgroundClass *background_class;
 
-	object_class = GTK_OBJECT_CLASS (klass);
+	object_class = G_OBJECT_CLASS (klass);
 	background_class = EEL_BACKGROUND_CLASS (klass);
 
 	signals[APPEARANCE_CHANGED] =
 		gtk_signal_new ("appearance_changed",
 				GTK_RUN_LAST | GTK_RUN_NO_RECURSE,
-				object_class->type,
+				GTK_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (EelBackgroundClass,
 						   appearance_changed),
-				gtk_marshal_NONE__NONE,
-				GTK_TYPE_NONE,
+				gtk_marshal_VOID__VOID,
+				G_TYPE_NONE,
 				0);
 	signals[SETTINGS_CHANGED] =
 		gtk_signal_new ("settings_changed",
 				GTK_RUN_LAST | GTK_RUN_NO_RECURSE,
-				object_class->type,
+				GTK_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (EelBackgroundClass,
 						   settings_changed),
-				gtk_marshal_NONE__NONE,
-				GTK_TYPE_NONE,
+				gtk_marshal_VOID__VOID,
+				G_TYPE_NONE,
 				0);
 	signals[IMAGE_LOADING_DONE] =
 		gtk_signal_new ("image_loading_done",
 				GTK_RUN_LAST | GTK_RUN_NO_RECURSE,
-				object_class->type,
+				GTK_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (EelBackgroundClass,
 						   image_loading_done),
-				gtk_marshal_NONE__BOOL,
-				GTK_TYPE_NONE,
+				gtk_marshal_VOID__BOOLEAN,
+				G_TYPE_NONE,
 				1,
-				GTK_TYPE_BOOL);
+				G_TYPE_BOOLEAN);
 	signals[RESET] =
 		gtk_signal_new ("reset",
 				GTK_RUN_LAST | GTK_RUN_NO_RECURSE,
-				object_class->type,
+				GTK_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (EelBackgroundClass,
 						   reset),
-				gtk_marshal_NONE__NONE,
-				GTK_TYPE_NONE,
+				gtk_marshal_VOID__VOID,
+				G_TYPE_NONE,
 				0);
 
-	gtk_object_class_add_signals (object_class, signals, LAST_SIGNAL);
-
-	object_class->destroy = eel_background_destroy;
+	object_class->finalize = eel_background_finalize;
 }
 
 static void
-eel_background_initialize (gpointer object, gpointer klass)
+eel_background_init (gpointer object, gpointer klass)
 {
 	EelBackground *background;
 
@@ -177,13 +163,13 @@ static void
 eel_background_remove_current_image (EelBackground *background)
 {
 	if (background->details->image != NULL) {
-		gdk_pixbuf_unref (background->details->image);
+		g_object_unref (background->details->image);
 		background->details->image = NULL;
 	}
 }
 
 static void
-eel_background_destroy (GtkObject *object)
+eel_background_finalize (GObject *object)
 {
 	EelBackground *background;
 
@@ -198,7 +184,7 @@ eel_background_destroy (GtkObject *object)
 	eel_background_remove_current_image (background);
 	g_free (background->details);
 
-	EEL_CALL_PARENT (GTK_OBJECT_CLASS, destroy, (object));
+	EEL_CALL_PARENT (G_OBJECT_CLASS, finalize, (object));
 }
 
 EelBackgroundImagePlacement
@@ -238,16 +224,16 @@ eel_background_set_image_placement (EelBackground *background,
 {
 	if (eel_background_set_image_placement_no_emit (background, new_placement)) {
 		gtk_signal_emit (GTK_OBJECT (background),
-			 signals[SETTINGS_CHANGED]);
+				 signals[SETTINGS_CHANGED]);
 		gtk_signal_emit (GTK_OBJECT (background),
-			 signals[APPEARANCE_CHANGED]);
+				 signals[APPEARANCE_CHANGED]);
 	}
 }
 
 EelBackground *
 eel_background_new (void)
 {
-	return EEL_BACKGROUND (gtk_object_new (EEL_TYPE_BACKGROUND, NULL));
+	return EEL_BACKGROUND (g_object_new (EEL_TYPE_BACKGROUND, NULL));
 }
  
 static void
@@ -495,7 +481,7 @@ eel_background_ensure_image_scaled (EelBackground *background, int dest_width, i
 					reload_image = TRUE;
 				} else {
 					scaled_pixbuf = gdk_pixbuf_scale_simple (background->details->image, dest_width, dest_height, GDK_INTERP_BILINEAR);
-					gdk_pixbuf_unref (background->details->image);
+					g_object_unref (background->details->image);
 					background->details->image = scaled_pixbuf;
 					image_width = gdk_pixbuf_get_width (scaled_pixbuf);
 					image_height = gdk_pixbuf_get_height (scaled_pixbuf);
@@ -512,7 +498,7 @@ eel_background_ensure_image_scaled (EelBackground *background, int dest_width, i
 					reload_image = TRUE;
 				} else {
 					scaled_pixbuf = eel_gdk_pixbuf_scale_to_fit (background->details->image, dest_width, dest_height);
-					gdk_pixbuf_unref (background->details->image);
+					g_object_unref (background->details->image);
 					background->details->image = scaled_pixbuf;
 					image_width = gdk_pixbuf_get_width (scaled_pixbuf);
 					image_height = gdk_pixbuf_get_height (scaled_pixbuf);
@@ -522,9 +508,9 @@ eel_background_ensure_image_scaled (EelBackground *background, int dest_width, i
 		}
 
 		if (reload_image) {
-			gdk_pixbuf_unref (background->details->image);
+			g_object_unref (background->details->image);
 			background->details->image = NULL;
-			eel_background_start_loading_image (background, TRUE);
+			eel_background_start_loading_image (background, TRUE, TRUE);
 			background->details->image_rect_x = 0;
 			background->details->image_rect_y = 0;
 			background->details->image_rect_width = 0;
@@ -568,7 +554,7 @@ eel_background_draw (EelBackground *background,
 	 * and passing it to the aa code.
 	 *
 	 * The width and height were chosen to match those used by gnome-canvas
-	 * (IMAGE_WIDTH_AA & IMAGE_HEIGHT_AA  in gnome-libs/libgnomeui/gnome-canvas.c)
+	 * (IMAGE_WIDTH_AA & IMAGE_HEIGHT_AA in libgnomecanvas/gnome-canvas.c)
 	 * They're not required to match - so this could be changed if necessary.
 	 */
 	static const int PIXBUF_WIDTH = 256;
@@ -597,7 +583,7 @@ eel_background_draw (EelBackground *background,
 		}
 	}
 	
-	gdk_pixbuf_unref (pixbuf);
+	g_object_unref (pixbuf);
 }
 
 void
@@ -782,13 +768,14 @@ eel_background_load_image_callback (GnomeVFSResult error,
 
 	/* Just ignore errors. */
 	if (pixbuf != NULL) {
-		gdk_pixbuf_ref (pixbuf);
+		g_object_ref (pixbuf);
 		background->details->image = pixbuf;
 		background->details->image_width_unscaled = gdk_pixbuf_get_width (pixbuf);
 		background->details->image_height_unscaled = gdk_pixbuf_get_height (pixbuf);
 	}
 
-	gtk_signal_emit (GTK_OBJECT (background), signals[IMAGE_LOADING_DONE], pixbuf != NULL || background->details->image_uri == NULL);
+	gtk_signal_emit (GTK_OBJECT (background), signals[IMAGE_LOADING_DONE],
+			 pixbuf != NULL || background->details->image_uri == NULL);
 
 	if (background->details->emit_after_load) {
 		gtk_signal_emit (GTK_OBJECT (background), signals[APPEARANCE_CHANGED]);
@@ -812,14 +799,21 @@ eel_background_cancel_loading_image (EelBackground *background)
 }
 
 static void
-eel_background_start_loading_image (EelBackground *background, gboolean emit_appearance_change)
+eel_background_start_loading_image (EelBackground *background, gboolean emit_appearance_change,
+				    gboolean load_async)
 {
 	background->details->emit_after_load = emit_appearance_change;
 
 	if (background->details->image_uri != NULL) {
-		background->details->load_image_handle = eel_gdk_pixbuf_load_async (background->details->image_uri,
-											 eel_background_load_image_callback,
-											 background);
+		if (load_async) {
+			background->details->load_image_handle = eel_gdk_pixbuf_load_async (background->details->image_uri,
+											    GNOME_VFS_PRIORITY_DEFAULT,
+											    eel_background_load_image_callback,
+											    background);
+		}
+		else {
+			eel_background_load_image_callback (0, eel_gdk_pixbuf_load (background->details->image_uri), background);
+		}
 	} else {
 		eel_background_load_image_callback (0, NULL, background);
 	}
@@ -829,7 +823,8 @@ static gboolean
 eel_background_set_image_uri_helper (EelBackground *background,
 					const char *image_uri,
 					gboolean emit_setting_change,
-					gboolean emit_appearance_change)
+					gboolean emit_appearance_change,
+				     gboolean load_async)
 {
 	g_return_val_if_fail (EEL_IS_BACKGROUND (background), FALSE);
 
@@ -847,8 +842,8 @@ eel_background_set_image_uri_helper (EelBackground *background,
 	 * current image can be used if an update is needed before the load completes.
 	 */
 	
-	eel_background_start_loading_image (background, emit_appearance_change);
-
+	eel_background_start_loading_image (background, emit_appearance_change, load_async);
+	
 	if (emit_setting_change) {
 		gtk_signal_emit (GTK_OBJECT (background), signals[SETTINGS_CHANGED]);
 	}
@@ -859,13 +854,19 @@ eel_background_set_image_uri_helper (EelBackground *background,
 void
 eel_background_set_image_uri (EelBackground *background, const char *image_uri)
 {
-	eel_background_set_image_uri_helper (background, image_uri, TRUE, TRUE);
+	eel_background_set_image_uri_helper (background, image_uri, TRUE, TRUE, TRUE);
+}
+
+void
+eel_background_set_image_uri_sync (EelBackground *background, const char *image_uri)
+{
+	eel_background_set_image_uri_helper (background, image_uri, TRUE, TRUE, FALSE);
 }
 
 static void
 set_image_and_color_image_loading_done_callback (EelBackground *background, gboolean successful_load, char *color)
 {
-	gtk_signal_disconnect_by_func (GTK_OBJECT (background), GTK_SIGNAL_FUNC (set_image_and_color_image_loading_done_callback), (gpointer) color);
+	gtk_signal_disconnect_by_func (GTK_OBJECT (background), G_CALLBACK (set_image_and_color_image_loading_done_callback), (gpointer) color);
 
 	eel_background_set_color_no_emit (background, color);
 
@@ -894,9 +895,9 @@ eel_background_set_image_uri_and_color (EelBackground *background, const char *i
 
 	color_copy = g_strdup (color);
 
-	gtk_signal_connect (GTK_OBJECT (background),
+	g_signal_connect (background,
 			    "image_loading_done",
-			    GTK_SIGNAL_FUNC (set_image_and_color_image_loading_done_callback),
+			    G_CALLBACK (set_image_and_color_image_loading_done_callback),
 			    (gpointer) color_copy);
 			    
 	/* set_image_and_color_image_loading_done_callback must always be called
@@ -911,7 +912,7 @@ eel_background_set_image_uri_and_color (EelBackground *background, const char *i
 	 * return value (if false) tells us whether or not we need to
 	 * call set_image_and_color_image_loading_done_callback ourselves.
 	 */
-	if (!eel_background_set_image_uri_helper (background, image_uri, FALSE, FALSE)) {
+	if (!eel_background_set_image_uri_helper (background, image_uri, FALSE, FALSE, TRUE)) {
 		set_image_and_color_image_loading_done_callback (background, TRUE, color_copy);
 	}
 }
@@ -920,149 +921,33 @@ void
 eel_background_receive_dropped_background_image (EelBackground *background,
 						      const char *image_uri)
 {
-	/* Currently, we only support tiled images. So we set the placement.
-	 * We rely on eel_background_set_image_uri_and_color to emit
-	 * the SETTINGS_CHANGED & APPEARANCE_CHANGE signals.
-	 */
-	eel_background_set_image_placement_no_emit (background, EEL_BACKGROUND_TILED);
+	/* Special case the reset-background image by file name. */
+	if (eel_str_has_suffix (image_uri, "/" RESET_BACKGROUND_MAGIC_IMAGE_NAME)) {
+		eel_background_reset (background);
+	} else {
+		/* Currently, we only support tiled images. So we set the placement.
+		 * We rely on eel_background_set_image_uri_and_color to emit
+		 * the SETTINGS_CHANGED & APPEARANCE_CHANGE signals.
+		 */
+		eel_background_set_image_placement_no_emit (background, EEL_BACKGROUND_TILED);
 
-	eel_background_set_image_uri_and_color (background, image_uri, NULL);
-}
-
-static GtkStyleClass *
-eel_gtk_style_get_default_class (void)
-{
-	static GtkStyleClass *default_class;
-	GtkStyle *style;
-	
-	if (default_class == NULL) {
-		style = gtk_style_new ();
-		default_class = style->klass;
-		gtk_style_unref (style);
+		eel_background_set_image_uri_and_color (background, image_uri, NULL);
 	}
-
-	return default_class;
-}
-
-static void
-eel_gdk_window_update_sizes (GdkWindow *window, int *width, int *height)
-{
-	g_return_if_fail (window != NULL);
-	g_return_if_fail (width != NULL);
-	g_return_if_fail (height != NULL);
-
-	if (*width == -1 && *height == -1) {
-		gdk_window_get_size (window, width, height);
-	} else if (*width == -1) {
-		gdk_window_get_size (window, width, NULL);
-	} else if (*height == -1) {
-		gdk_window_get_size (window, NULL, height);
-	}
-}
-
-static void
-eel_background_draw_flat_box (GtkStyle *style,
-				   GdkWindow *window,
-				   GtkStateType state_type,
-				   GtkShadowType shadow_type,
-				   GdkRectangle *area,
-				   GtkWidget *widget,
-				   char *detail,
-				   int x,
-				   int y,
-				   int width,
-				   int height)
-{
-	gboolean call_parent;
-	EelBackground *background;
-	GdkGC *gc;
-
-	call_parent = TRUE;
-
-	background = NULL;
-	if (state_type == GTK_STATE_NORMAL) {
-		background = eel_get_widget_background (widget);
-		if (background != NULL) {
-			call_parent = FALSE;
-		}
-	}
-
-	if (call_parent) {
-		(* eel_gtk_style_get_default_class()->draw_flat_box)
-			(style, window, state_type, shadow_type, area, widget,
-			 detail, x, y, width, height);
-		return;
-	}
-
-    	gc = style->bg_gc[state_type];
-	if (area)
-		gdk_gc_set_clip_rectangle (gc, area);
-
-	eel_gdk_window_update_sizes (window, &width, &height);	
-
-	if (area)
-		eel_background_draw_to_drawable (background, window, gc,
-						 area->x, area->y,
-						 area->width, area->height,
-						 widget->allocation.width, widget->allocation.height);
-	else
-		eel_background_draw_to_drawable (background, window, gc,
-						 0, 0,
-						 width, height,
-						 widget->allocation.width, widget->allocation.height);
-	if (area)
-		gdk_gc_set_clip_rectangle (gc, NULL);
-}
-
-static GtkStyleClass *
-eel_background_get_gtk_style_class (void)
-{
-	static GtkStyleClass *klass;
-
-	if (klass == NULL) {
-		static GtkStyleClass klass_storage;
-
-		klass = &klass_storage;
-		*klass = *eel_gtk_style_get_default_class ();
-
-		klass->draw_flat_box = eel_background_draw_flat_box;
-	}
-
-	return klass;
 }
 
 static void
 eel_background_set_widget_style (EelBackground *background,
-				      GtkWidget *widget)
+				 GtkWidget *widget)
 {
 	GtkStyle *style;
-	char *start_color_spec;
-	GdkColor color;
 	
 	g_return_if_fail (EEL_IS_BACKGROUND (background));
 	g_return_if_fail (GTK_IS_WIDGET (widget));
 	
-	style = gtk_widget_get_style (widget);
-	
-	/* Make a copy of the style. */
-	style = gtk_style_copy (style);
-	
-	/* Give it the special class that allows it to draw gradients. */
-	style->klass = eel_background_get_gtk_style_class ();
-
-	/* Set up the colors in the style. */
-	start_color_spec = eel_gradient_get_start_color_spec (background->details->color);
-	eel_gdk_color_parse_with_white_default (start_color_spec, &color);
-	g_free (start_color_spec);
-	
-	style->bg[GTK_STATE_NORMAL] = color;
-	style->base[GTK_STATE_NORMAL] = color;
-	style->bg[GTK_STATE_ACTIVE] = color;
-	style->base[GTK_STATE_ACTIVE] = color;
-	
-	/* Put the style in the widget. */
+	style = GTK_STYLE (eel_background_style_new (gtk_widget_get_style (widget),
+						     background));	
 	gtk_widget_set_style (widget, style);
-	gtk_style_unref (style);
+	g_object_unref (style);
 }
 
 /**
@@ -1105,12 +990,73 @@ eel_background_reset (EelBackground *background)
 	gtk_signal_emit (GTK_OBJECT (background), signals[RESET]);
 }
 
+static void
+draw_background_callback (GnomeCanvas *canvas, GdkDrawable *drawable,
+			  int x, int y, int width, int height)
+{
+	EelBackground *background;
+	GdkGC *gc;
+
+	background = eel_get_widget_background (GTK_WIDGET (canvas));
+	if (background == NULL) {
+		return;
+	}
+
+	eel_background_pre_draw (background,
+				 GTK_WIDGET (canvas)->allocation.width,
+				 GTK_WIDGET (canvas)->allocation.height);
+
+	/* Create a new gc each time.
+	 * If this is a speed problem, we can create one and keep it around,
+	 * but it's a bit more complicated to ensure that it's always compatible
+	 * with whatever drawable is passed in.
+	 */
+	gc = gdk_gc_new (drawable);
+	eel_background_draw (background, drawable, gc, x, y, 0, 0, width, height);
+	gdk_gc_unref (gc);
+
+	/* We don't want the draw from the base class to happen. */
+	g_signal_stop_emission_by_name (canvas, "draw_background");
+}
+
+static void
+render_background_callback (GnomeCanvas *canvas, GnomeCanvasBuf *buffer)
+{
+	EelBackground *background;
+			
+	background = eel_get_widget_background (GTK_WIDGET (canvas));
+	if (background == NULL) {
+		return;
+	}
+
+	eel_background_pre_draw (background,
+				 GTK_WIDGET (canvas)->allocation.width,
+				 GTK_WIDGET (canvas)->allocation.height);
+
+	eel_background_draw_aa (background, buffer);
+	
+	/* We don't want the render from the base class to happen. */
+	g_signal_stop_emission_by_name (canvas, "render_background");
+}
+
 static void
 eel_background_set_up_canvas (GtkWidget *widget)
 {
-	if (GNOME_IS_CANVAS (widget)) {
-		eel_background_canvas_group_supplant_root_class (GNOME_CANVAS (widget));
+	if (!GNOME_IS_CANVAS (widget)) {
+		return;
 	}
+	if (g_object_get_data (G_OBJECT (widget), "eel_background_canvas_is_set_up") != NULL) {
+		return;
+	}
+
+	gnome_canvas_set_dither (GNOME_CANVAS (widget), GDK_RGB_DITHER_MAX);
+
+	g_signal_connect (widget, "draw_background",
+			  G_CALLBACK (draw_background_callback), NULL);
+	g_signal_connect (widget, "render_background",
+			  G_CALLBACK (render_background_callback), NULL);
+
+	g_object_set_data (G_OBJECT (widget), "eel_background_canvas_is_set_up", widget);
 }
 
 static void
@@ -1119,7 +1065,7 @@ eel_widget_background_changed (GtkWidget *widget, EelBackground *background)
 	eel_background_set_widget_style (background, widget);
 	eel_background_set_up_canvas (widget);
 
-	gtk_widget_queue_clear (widget);
+	gtk_widget_queue_draw (widget);
 }
 
 /* Gets the background attached to a widget.
@@ -1141,7 +1087,7 @@ eel_get_widget_background (GtkWidget *widget)
 	g_return_val_if_fail (GTK_IS_WIDGET (widget), NULL);
 
 	/* Check for an existing background. */
-	data = gtk_object_get_data (GTK_OBJECT (widget), "eel_background");
+	data = g_object_get_data (G_OBJECT (widget), "eel_background");
 	if (data != NULL) {
 		g_assert (EEL_IS_BACKGROUND (data));
 		return data;
@@ -1149,15 +1095,15 @@ eel_get_widget_background (GtkWidget *widget)
 
 	/* Store the background in the widget's data. */
 	background = eel_background_new ();
-	gtk_object_ref (GTK_OBJECT (background));
+	g_object_ref (background);
 	gtk_object_sink (GTK_OBJECT (background));
-	gtk_object_set_data_full (GTK_OBJECT (widget), "eel_background",
-				  background, (GtkDestroyNotify) gtk_object_unref);
+	g_object_set_data_full (G_OBJECT (widget), "eel_background",
+				background, (GtkDestroyNotify) g_object_unref);
 
 	/* Arrange to get the signal whenever the background changes. */
 	gtk_signal_connect_object_while_alive (GTK_OBJECT (background),
 					       "appearance_changed",
-					       eel_widget_background_changed,
+					       G_CALLBACK (eel_widget_background_changed),
 					       GTK_OBJECT (widget));
 	eel_widget_background_changed (widget, background);
 
@@ -1169,7 +1115,7 @@ eel_widget_has_attached_background (GtkWidget *widget)
 {
 	g_return_val_if_fail (GTK_IS_WIDGET (widget), FALSE);
 
-	return gtk_object_get_data (GTK_OBJECT (widget), "eel_background") != NULL;
+	return g_object_get_data (G_OBJECT (widget), "eel_background") != NULL;
 }
 
 GtkWidget *
@@ -1236,7 +1182,7 @@ eel_background_is_dark (EelBackground *background)
 		color.blue = (color.blue + end_color.blue) / 2;
 	}
 	if (background->details->image != NULL) {
-		argb = eel_gdk_pixbuf_average_value_argb (background->details->image);
+		argb = eel_gdk_pixbuf_average_value (background->details->image);
 		
 		a = argb >> 24;
 
@@ -1252,10 +1198,10 @@ eel_background_is_dark (EelBackground *background)
 /* handle dropped colors */
 void
 eel_background_receive_dropped_color (EelBackground *background,
-					   GtkWidget *widget,
-					   int drop_location_x,
-					   int drop_location_y,
-					   const GtkSelectionData *selection_data)
+				      GtkWidget *widget,
+				      int drop_location_x,
+				      int drop_location_y,
+				      const GtkSelectionData *selection_data)
 {
 	guint16 *channels;
 	char *color_spec;
@@ -1272,7 +1218,10 @@ eel_background_receive_dropped_color (EelBackground *background,
 		return;
 	}
 	channels = (guint16 *) selection_data->data;
-	color_spec = g_strdup_printf ("rgb:%04hX/%04hX/%04hX", channels[0], channels[1], channels[2]);
+	color_spec = g_strdup_printf ("#%02X%02X%02X",
+				      channels[0] >> 8,
+				      channels[1] >> 8,
+				      channels[2] >> 8);
 
 	/* Figure out if the color was dropped close enough to an edge to create a gradient.
 	   For the moment, this is hard-wired, but later the widget will have to have some
@@ -1318,7 +1267,7 @@ eel_self_check_background (void)
 	eel_background_set_color (background, "red-blue");
 	eel_background_set_color (background, "red-blue:h");
 
-	gtk_object_unref (GTK_OBJECT (background));
+	gtk_object_sink (GTK_OBJECT (background));
 }
 
 #endif /* !EEL_OMIT_SELF_CHECK */
