@@ -871,3 +871,39 @@ eel_gnome_canvas_get_pango_context (GnomeCanvas *canvas)
 		return eel_gtk_widget_get_pango_ft2_context (GTK_WIDGET (canvas));
 	}
 }
+
+void
+eel_gnome_canvas_item_send_behind (GnomeCanvasItem *item,
+				   GnomeCanvasItem *behind_item)
+{
+	GList *item_list;
+	int item_position, behind_position;
+
+	g_return_if_fail (GNOME_IS_CANVAS_ITEM (item));
+
+	if (behind_item == NULL) {
+		gnome_canvas_item_raise_to_top (item);
+		return;
+	}
+
+	g_return_if_fail (GNOME_IS_CANVAS_ITEM (behind_item));
+	g_return_if_fail (item->parent == behind_item->parent);
+
+	item_list = GNOME_CANVAS_GROUP (item->parent)->item_list;
+
+	item_position = g_list_index (item_list, item);
+	g_assert (item_position != -1);
+	behind_position = g_list_index (item_list, behind_item);
+	g_assert (behind_position != -1);
+	g_assert (item_position != behind_position);
+
+	if (item_position == behind_position - 1) {
+		return;
+	}
+
+	if (item_position < behind_position) {
+		gnome_canvas_item_raise (item, (behind_position - 1) - item_position);
+	} else {
+		gnome_canvas_item_lower (item, item_position - behind_position);
+	}
+}
