@@ -23,17 +23,18 @@
 */
 
 #include <config.h>
-
 #include "eel-image-table.h"
 
-#include "eel-gtk-macros.h"
-#include "eel-gtk-extensions.h"
 #include "eel-art-extensions.h"
 #include "eel-art-gtk-extensions.h"
-#include "eel-region.h"
 #include "eel-debug-drawing.h"
-
+#include "eel-gtk-extensions.h"
+#include "eel-gtk-macros.h"
+#include "eel-labeled-image.h"
+#include "eel-marshal.h"
+#include "eel-region.h"
 #include <gtk/gtkmain.h>
+#include <gtk/gtksignal.h>
 
 /* Arguments */
 enum
@@ -67,10 +68,11 @@ typedef enum
 /* Signals */
 static guint image_table_signals[LAST_SIGNAL] = { 0 };
 
-/* GtkObjectClass methods */
-static void    eel_image_table_initialize_class     (EelImageTableClass *image_table_class);
-static void    eel_image_table_initialize           (EelImageTable      *image);
-static void    eel_image_table_destroy              (GtkObject          *object);
+static void    eel_image_table_class_init     (EelImageTableClass *image_table_class);
+static void    eel_image_table_init           (EelImageTable      *image);
+
+/* GObjectClass methods */
+static void    eel_image_table_finalize             (GObject            *object);
 
 /* GtkWidgetClass methods */
 static int     eel_image_table_expose_event         (GtkWidget          *widget,
@@ -111,22 +113,17 @@ static int     ancestor_button_release_event        (GtkWidget          *widget,
 						     GdkEventButton     *event,
 						     gpointer            event_data);
 
-/* EelImageTable signals */
-static void    eel_image_table_set_is_smooth_signal (GtkWidget          *widget,
-						     gboolean            is_smooth);
+EEL_CLASS_BOILERPLATE (EelImageTable, eel_image_table, EEL_TYPE_WRAP_TABLE)
 
-EEL_DEFINE_CLASS_BOILERPLATE (EelImageTable, eel_image_table, EEL_TYPE_WRAP_TABLE)
-
-/* Class init methods */
-	static void
-eel_image_table_initialize_class (EelImageTableClass *image_table_class)
+static void
+eel_image_table_class_init (EelImageTableClass *image_table_class)
 {
-	GtkObjectClass *object_class = GTK_OBJECT_CLASS (image_table_class);
+	GObjectClass *object_class = G_OBJECT_CLASS (image_table_class);
 	GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (image_table_class);
 	GtkContainerClass *container_class = GTK_CONTAINER_CLASS (image_table_class);
 
-	/* GtkObjectClass */
-	object_class->destroy = eel_image_table_destroy;
+	/* GObjectClass */
+	object_class->finalize = eel_image_table_finalize;
 	
  	/* GtkWidgetClass */
  	widget_class->expose_event = eel_image_table_expose_event;
@@ -137,96 +134,74 @@ eel_image_table_initialize_class (EelImageTableClass *image_table_class)
 	container_class->remove = eel_image_table_remove;
 	container_class->child_type = eel_image_table_child_type;
 
-	/* EelImageTableClass */
-	image_table_class->set_is_smooth = eel_image_table_set_is_smooth_signal;
-	
 	/* Signals */
 	image_table_signals[CHILD_ENTER] = gtk_signal_new ("child_enter",
 							   GTK_RUN_LAST,
-							   object_class->type,
-							   0,
-							   gtk_marshal_NONE__POINTER_POINTER,
-							   GTK_TYPE_NONE,
+							   GTK_CLASS_TYPE (object_class),
+							   G_STRUCT_OFFSET (EelImageTableClass, child_enter),
+							   eel_marshal_VOID__OBJECT_POINTER,
+							   G_TYPE_NONE,
 							   2,
-							   GTK_TYPE_POINTER,
-							   GTK_TYPE_POINTER);
+							   GTK_TYPE_WIDGET,
+							   G_TYPE_POINTER);
 	image_table_signals[CHILD_LEAVE] = gtk_signal_new ("child_leave",
 							   GTK_RUN_LAST,
-							   object_class->type,
-							   0,
-							   gtk_marshal_NONE__POINTER_POINTER,
-							   GTK_TYPE_NONE,
+							   GTK_CLASS_TYPE (object_class),
+							   G_STRUCT_OFFSET (EelImageTableClass, child_leave),
+							   eel_marshal_VOID__OBJECT_POINTER,
+							   G_TYPE_NONE,
 							   2,
-							   GTK_TYPE_POINTER,
-							   GTK_TYPE_POINTER);
+							   GTK_TYPE_WIDGET,
+							   G_TYPE_POINTER);
 	image_table_signals[CHILD_PRESSED] = gtk_signal_new ("child_pressed",
 							     GTK_RUN_LAST,
-							     object_class->type,
-							     0,
-							     gtk_marshal_NONE__POINTER_POINTER,
-							     GTK_TYPE_NONE,
+							     GTK_CLASS_TYPE (object_class),
+							     G_STRUCT_OFFSET (EelImageTableClass, child_pressed),
+							     eel_marshal_VOID__OBJECT_POINTER,
+							     G_TYPE_NONE,
 							     2,
-							     GTK_TYPE_POINTER,
-							     GTK_TYPE_POINTER);
+							     GTK_TYPE_WIDGET,
+							     G_TYPE_POINTER);
 	image_table_signals[CHILD_RELEASED] = gtk_signal_new ("child_released",
 							      GTK_RUN_LAST,
-							      object_class->type,
-							      0,
-							      gtk_marshal_NONE__POINTER_POINTER,
-							      GTK_TYPE_NONE,
+							      GTK_CLASS_TYPE (object_class),
+							      G_STRUCT_OFFSET (EelImageTableClass, child_released),
+							      eel_marshal_VOID__OBJECT_POINTER,
+							      G_TYPE_NONE,
 							      2,
-							      GTK_TYPE_POINTER,
-							      GTK_TYPE_POINTER);
+							      GTK_TYPE_WIDGET,
+							      G_TYPE_POINTER);
 	image_table_signals[CHILD_CLICKED] = gtk_signal_new ("child_clicked",
 							     GTK_RUN_LAST,
-							     object_class->type,
-							     0,
-							     gtk_marshal_NONE__POINTER_POINTER,
-							     GTK_TYPE_NONE,
+							     GTK_CLASS_TYPE (object_class),
+							     G_STRUCT_OFFSET (EelImageTableClass, child_clicked),
+							     eel_marshal_VOID__OBJECT_POINTER,
+							     G_TYPE_NONE,
 							     2,
-							     GTK_TYPE_POINTER,
-							     GTK_TYPE_POINTER);
-	
-	image_table_signals[SET_IS_SMOOTH] = gtk_signal_new ("set_is_smooth",
-							     GTK_RUN_LAST,
-							     object_class->type,
-							     GTK_SIGNAL_OFFSET (EelImageTableClass, set_is_smooth),
-							     gtk_marshal_NONE__BOOL,
-							     GTK_TYPE_NONE, 
-							     1,
-							     GTK_TYPE_BOOL);
-
-	gtk_object_class_add_signals (object_class, image_table_signals, LAST_SIGNAL);
-
-	/* Let the smooth widget machinery know that our class can be smooth */
-	eel_smooth_widget_register_type (EEL_TYPE_IMAGE_TABLE);
+							     GTK_TYPE_WIDGET,
+							     G_TYPE_POINTER);
 }
 
 void
-eel_image_table_initialize (EelImageTable *image_table)
+eel_image_table_init (EelImageTable *image_table)
 {
 	GTK_WIDGET_SET_FLAGS (image_table, GTK_NO_WINDOW);
 
 	image_table->details = g_new0 (EelImageTableDetails, 1);
 	image_table->details->smooth_background_color = EEL_RGB_COLOR_WHITE;
-
-	eel_smooth_widget_register (GTK_WIDGET (image_table));
 }
 
-/* GtkObjectClass methods */
+/* GObjectClass methods */
 static void
-eel_image_table_destroy (GtkObject *object)
+eel_image_table_finalize (GObject *object)
 {
  	EelImageTable *image_table;
 	
-	g_return_if_fail (EEL_IS_IMAGE_TABLE (object));
-
 	image_table = EEL_IMAGE_TABLE (object);
 
 	g_free (image_table->details);
 
-	/* Chain destroy */
-	EEL_CALL_PARENT (GTK_OBJECT_CLASS, destroy, (object));
+	EEL_CALL_PARENT (G_OBJECT_CLASS, finalize, (object));
 }
 
 /* GtkWidgetClass methods */
@@ -279,31 +254,31 @@ eel_image_table_realize (GtkWidget *widget)
 
 	eel_gtk_signal_connect_while_realized (GTK_OBJECT (windowed_ancestor),
 					       "enter_notify_event",
-					       GTK_SIGNAL_FUNC (ancestor_enter_notify_event),
+					       G_CALLBACK (ancestor_enter_notify_event),
 					       widget,
 					       widget);
 	
 	eel_gtk_signal_connect_while_realized (GTK_OBJECT (windowed_ancestor),
 					       "leave_notify_event",
-					       GTK_SIGNAL_FUNC (ancestor_leave_notify_event),
+					       G_CALLBACK (ancestor_leave_notify_event),
 					       widget,
 					       widget);
 	
 	eel_gtk_signal_connect_while_realized (GTK_OBJECT (windowed_ancestor),
 					       "motion_notify_event",
-					       GTK_SIGNAL_FUNC (ancestor_motion_notify_event),
+					       G_CALLBACK (ancestor_motion_notify_event),
 					       widget,
 					       widget);
 	
 	eel_gtk_signal_connect_while_realized (GTK_OBJECT (windowed_ancestor),
 					       "button_press_event",
-					       GTK_SIGNAL_FUNC (ancestor_button_press_event),
+					       G_CALLBACK (ancestor_button_press_event),
 					       widget,
 					       widget);
 	
 	eel_gtk_signal_connect_while_realized (GTK_OBJECT (windowed_ancestor),
 					       "button_release_event",
-					       GTK_SIGNAL_FUNC (ancestor_button_release_event),
+					       G_CALLBACK (ancestor_button_release_event),
 					       widget,
 					       widget);
 }
@@ -355,16 +330,6 @@ eel_image_table_child_type (GtkContainer *container)
 	return EEL_TYPE_LABELED_IMAGE;
 }
 
-/* EelImageTable signals */
-static void
-eel_image_table_set_is_smooth_signal (GtkWidget *widget,
-				      gboolean is_smooth)
-{
-	g_return_if_fail (EEL_IS_IMAGE_TABLE (widget));
-	
-	eel_image_table_set_is_smooth (EEL_IMAGE_TABLE (widget), is_smooth);
-}
-
 /* Private EelImageTable methods */
 
 /* For each of the image table children, subtract their content from a region */
@@ -470,10 +435,11 @@ image_table_emit_signal (EelImageTable *image_table,
 	event.button = button;
 	event.state = state;
 	
-	gtk_signal_emit (GTK_OBJECT (image_table), 
-			 image_table_signals[signal_index],
-			 child,
-			 &event);
+	g_signal_emit (image_table, 
+		       image_table_signals[signal_index],
+		       0,
+		       child,
+		       &event);
 }
 
 static void
@@ -696,25 +662,6 @@ eel_image_table_new (gboolean homogeneous)
 	return GTK_WIDGET (image_table);
 }
 
-/**
- * eel_image_table_set_is_smooth:
- * @image_table: A EelImageTable.
- * @is_smooth: Boolean value indicating whether the image table is smooth.
- *
- */
-void
-eel_image_table_set_is_smooth (EelImageTable *image_table,
-			       gboolean is_smooth)
-{
-	g_return_if_fail (EEL_IS_IMAGE_TABLE (image_table));
-	
-	if (image_table->details->is_smooth == is_smooth) {
-		return;
-	}
-	
-	image_table->details->is_smooth = is_smooth;
-}
-
 /**
  * eel_image_table_set_smooth_background_color:
  * @image_table: A EelImageTable.
