@@ -105,38 +105,48 @@ struct EelListColumnTitleDetails
 
 };
 
-static void eel_list_column_title_initialize_class	(gpointer klass);
-static void eel_list_column_title_initialize       (gpointer object, gpointer klass);
-static void eel_list_column_title_paint		(GtkWidget *widget, GtkWidget *draw_target, GdkDrawable *target_drawable, GdkRectangle *area);
-static void eel_list_column_title_draw 		(GtkWidget *widget, GdkRectangle *box);
-static void eel_list_column_title_buffered_draw	(GtkWidget *widget);
-static void eel_list_column_title_queue_buffered_draw(GtkWidget *widget);
-static gboolean eel_list_column_title_expose 	(GtkWidget *widget, GdkEventExpose *event);
-static void eel_list_column_title_realize 		(GtkWidget *widget);
-static void eel_list_column_title_finalize 	(GtkObject *object);
-static void eel_list_column_title_request 		(GtkWidget *widget, GtkRequisition *requisition);
+static void     eel_list_column_title_class_init    (gpointer          klass);
+static void     eel_list_column_title_init          (gpointer          object,
+							   gpointer          klass);
+static void     eel_list_column_title_paint               (GtkWidget        *widget,
+							   GtkWidget        *draw_target,
+							   GdkDrawable      *target_drawable,
+							   GdkRectangle     *area);
+static void     eel_list_column_title_buffered_draw       (GtkWidget        *widget);
+static void     eel_list_column_title_queue_buffered_draw (GtkWidget        *widget);
+static gboolean eel_list_column_title_expose              (GtkWidget        *widget,
+							   GdkEventExpose   *event);
+static void     eel_list_column_title_realize             (GtkWidget        *widget);
+static void     eel_list_column_title_finalize            (GObject          *object);
+static void     eel_list_column_title_request             (GtkWidget        *widget,
+							   GtkRequisition   *requisition);
+static gboolean eel_list_column_title_motion              (GtkWidget        *widget,
+							   GdkEventMotion   *event);
+static gboolean eel_list_column_title_leave               (GtkWidget        *widget,
+							   GdkEventCrossing *event);
+static gboolean eel_list_column_title_button_press        (GtkWidget        *widget,
+							   GdkEventButton   *event);
+static gboolean eel_list_column_title_button_release      (GtkWidget        *widget,
+							   GdkEventButton   *event);
 
-static gboolean eel_list_column_title_motion	(GtkWidget *widget, GdkEventMotion *event);
-static gboolean eel_list_column_title_leave 	(GtkWidget *widget, GdkEventCrossing *event);
-
-static gboolean eel_list_column_title_button_press	(GtkWidget *widget, GdkEventButton *event);
-static gboolean eel_list_column_title_button_release (GtkWidget *widget, GdkEventButton *event);
-
-
-EEL_DEFINE_CLASS_BOILERPLATE (EelListColumnTitle, eel_list_column_title, GTK_TYPE_BIN)
+EEL_CLASS_BOILERPLATE (EelListColumnTitle,
+		       eel_list_column_title,
+		       GTK_TYPE_BIN)
 /* generates eel_list_column_title_get_type */
 
 static void
-eel_list_column_title_initialize_class (gpointer klass)
+eel_list_column_title_class_init (gpointer klass)
 {
+	GObjectClass *gobject_class;
 	GtkObjectClass *object_class;
 	GtkWidgetClass *widget_class;
 
+	gobject_class = G_OBJECT_CLASS (klass);
 	object_class = GTK_OBJECT_CLASS (klass);
 	widget_class = GTK_WIDGET_CLASS (klass);
 
-	object_class->finalize = eel_list_column_title_finalize;
-	widget_class->draw = eel_list_column_title_draw;
+	gobject_class->finalize = eel_list_column_title_finalize;
+
 	widget_class->expose_event = eel_list_column_title_expose;
 	widget_class->realize = eel_list_column_title_realize;
 	widget_class->size_request = eel_list_column_title_request;
@@ -154,7 +164,7 @@ eel_list_column_title_new (void)
 }
 
 static void
-eel_list_column_title_initialize (gpointer object, gpointer klass)
+eel_list_column_title_init (gpointer object, gpointer klass)
 {
 	EelListColumnTitle *column_title;
 
@@ -220,11 +230,11 @@ eel_list_column_title_realize (GtkWidget *widget)
 }
 
 static void
-eel_list_column_title_finalize (GtkObject *object)
+eel_list_column_title_finalize (GObject *object)
 {
 	EelListColumnTitle *column_title;
 
-	column_title = EEL_LIST_COLUMN_TITLE(object);
+	column_title = EEL_LIST_COLUMN_TITLE (object);
 
 	if (column_title->details->up_indicator_pixmap != NULL) {
 		gdk_pixmap_unref (column_title->details->up_indicator_pixmap);
@@ -250,11 +260,12 @@ eel_list_column_title_finalize (GtkObject *object)
 
 	if (column_title->details->copy_area_gc != NULL) {
 		gdk_gc_destroy (column_title->details->copy_area_gc);
+		column_title->details->copy_area_gc = NULL;
 	}
 
 	g_free (column_title->details);
 	
-	EEL_CALL_PARENT (GTK_OBJECT_CLASS, finalize, (object));
+	EEL_CALL_PARENT (G_OBJECT_CLASS, finalize, (object));
 }
 
 static void
@@ -266,9 +277,9 @@ eel_list_column_title_request (GtkWidget *widget, GtkRequisition *requisition)
 	g_assert (requisition != NULL);
 
 	requisition->width = (GTK_CONTAINER (widget)->border_width + CHILD_SPACING +
-			     widget->style->klass->xthickness) * 2;
+			     widget->style->xthickness) * 2;
 	requisition->height = (GTK_CONTAINER (widget)->border_width + CHILD_SPACING +
-			      widget->style->klass->ythickness) * 2;
+			      widget->style->ythickness) * 2;
 
 
 	if (GTK_BIN (widget)->child && GTK_WIDGET_VISIBLE (GTK_BIN (widget)->child)) {
@@ -327,7 +338,7 @@ load_up_indicator (const char **xpm_data,
 
 	gdk_pixbuf_render_pixmap_and_mask (pixbuf, indicator_pixmap, indicator_mask, EEL_STANDARD_ALPHA_THRESHHOLD);
 
-	gdk_pixbuf_unref (pixbuf);
+	g_object_unref (pixbuf);
 }
 
 static void
@@ -543,6 +554,7 @@ eel_list_column_title_paint (GtkWidget *widget, GtkWidget *draw_target,
 			char *truncated_label;
 			int truncanted_width;
 			GdkRectangle temporary;
+			GdkFont *gdk_font;
 
 			/* Extend the redraw area vertically to contain the entire cell 
 			 * -- seems like if I don't do this, for short exposed areas no text
@@ -561,7 +573,9 @@ eel_list_column_title_paint (GtkWidget *widget, GtkWidget *draw_target,
 			eel_rectangle_inset (&temporary, 2, 2);
 			gdk_rectangle_intersect (&cell_redraw_area, &temporary, &cell_redraw_area);
 
-			truncated_label = truncate_string (cell_label, widget->style->font, 
+			/* FIXME: ugly hack for now ! */
+			gdk_font = gtk_style_get_font (widget->style);
+			truncated_label = truncate_string (cell_label, gdk_font,
 				text_x_available_end - text_x_offset, &truncanted_width);
 		
 			if (right_justified) {
@@ -578,19 +592,6 @@ eel_list_column_title_paint (GtkWidget *widget, GtkWidget *draw_target,
 	}
 }
 
-static void
-eel_list_column_title_draw (GtkWidget *widget, GdkRectangle *area)
-{
-	g_assert (EEL_IS_LIST_COLUMN_TITLE (widget));
-	g_assert (area != NULL);
-
-	if (!GTK_WIDGET_DRAWABLE (widget)) {
-		return;
-	}
-
-	eel_list_column_title_paint (widget, widget, widget->window, area);
-}
-
 static void
 eel_list_column_title_buffered_draw (GtkWidget *widget)
 {
@@ -1002,7 +1003,7 @@ eel_list_column_title_button_release (GtkWidget *widget, GdkEventButton *event)
 	} else if (column_title->details->tracking_column_press != -1) {
 
 		/* column title press -- change the sort order */
-		gtk_signal_emit_by_name (GTK_OBJECT (parent_list), "click_column", 
+		g_signal_emit_by_name (parent_list, "click_column", 
 					 column_title->details->tracking_column_press);
 		/* end press tracking */
 		column_title->details->tracking_column_press = -1;
